# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0011_auto_20150425_1710'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='interval_restriction',
            field=models.PositiveIntegerField(default=None, help_text=b'Interval before script can be run again', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='mmcscript',
            name='enable_queries',
            field=models.BooleanField(default=False, help_text=b'Profile queries numbers.'),
        ),
        migrations.AlterField(
            model_name='mmcscript',
            name='trigger_time',
            field=models.FloatField(help_text=b'Set the threshold sec for execution time', null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0007_mmcscript_calls'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='real_time',
            field=models.BooleanField(default=False, help_text=b'Real Time info about command (for long tasks).'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0006_auto_20150424_2114'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='calls',
            field=models.BigIntegerField(default=0, verbose_name=b'Number of calls'),
        ),
    ]



__author__ = 'gotlium'

import time
from django.core.management.base import NoArgsCommand


class Command(NoArgsCommand):
    def handle_noargs(self, *args, **options):
        time.sleep(4)
        print("OK.")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0005_auto_20150315_0644'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmclog',
            name='pid',
            field=models.IntegerField(default=1),
        ),
        migrations.AlterField(
            model_name='mmcemail',
            name='email',
            field=models.EmailField(help_text=b'Email will be used for send all exceptions from command.', max_length=254),
        ),
    ]



# Django settings for demo project.
import sys

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:' if 'test' in sys.argv else 'db.sqlite',
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    }
}

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = []

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = ''

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'iq*u@1!or^52d4qg7d54gjne$7fd@r=8jxu00(r24648wo5p(*'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'demo.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'demo.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

from django import VERSION

if VERSION[:2] < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    # 'south',
    'mmc',
    'apps.test',
)

# If you have multiple cron instances, you can specify RedisLock or
# MemcacheLock for run only one copy of management command.
MMC_LOCK_TYPE = 'FileLock'

# Settings for RedisLock
# MMC_REDIS_CONFIG = {
#     'host': 'localhost', 'port': 6379,
#     'db': 0, 'password': None
# }

# Settings for MemcacheLock
# MMC_MEMCACHED_CONFIG = {
#     'servers': ['127.0.0.1:11211'], 'debug': 0
# }

# Mail subject, for send any errors on your script
MMC_SUBJECT = '[MMC] Errors'

# Enable sentry notification
MMC_SENTRY_NOTIFICATION = True

# Enable email notifications
MMC_EMAIL_NOTIFICATION = True

try:
    from local_settings import *
except ImportError:
    pass



# -*- encoding: utf-8 -*-

__author__ = 'gotlium'

import optparse
import datetime

from django.core.management.base import BaseCommand
from django.utils.timezone import now

from mmc.models import MMCHost, MMCLog, MMCScript
from mmc.defaults import CLEANUP_IGNORED


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        optparse.make_option(
            '-l', '--delta', action='store', dest='delta', default=0),
        optparse.make_option(
            '-d', '--date', action='store', dest='date', default=None),
    )

    def __init__(self):
        super(Command, self).__init__()
        self.days_delta = 0

    def _set_delta(self, options):
        self.days_delta = now() - datetime.timedelta(
            days=int(options.get('delta')))

        if options.get('date'):
            dt = datetime.datetime.strptime(options.get('date'), '%Y-%m-%d')
            self.days_delta = (now() - dt)

    def _cleanup_model(self, model):
        model.objects.filter(created__lte=self.days_delta).delete()

    @staticmethod
    def _cleanup_ignored():
        if CLEANUP_IGNORED is True:
            MMCLog.objects.filter(
                script__ignore=True, script__save_on_error=False).delete()

    def _cleanup(self):
        self._cleanup_model(MMCScript)
        self._cleanup_model(MMCHost)
        self._cleanup_model(MMCLog)
        self._cleanup_ignored()

    def handle(self, **options):
        self._set_delta(options)
        self._cleanup()



# -*- encoding: utf-8 -*-

__author__ = 'gotlium'

import pprint

from django.core.management.base import BaseCommand
from django.db import connections


class Command(BaseCommand):
    def handle(self, *args, **options):
        print("Запросы:")
        pprint.pprint(connections['default'].queries, indent=4)
        raise Exception(u"Ошибочка")



# -*- encoding: utf-8 -*-

__author__ = 'gotlium'

from django.core.management.base import BaseCommand
import mmc


class Command(BaseCommand):
    def raise_exception(self):
        if mmc.PY2 is True:
            raise Exception(u'Привет, мир!')
        raise Exception('Привет, мир!')

    def handle(self, *args, **options):
        self.raise_exception()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0012_auto_20150717_0649'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmclog',
            name='is_fixed',
            field=models.NullBooleanField(default=None),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MMCEmail',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateField(auto_now=True)),
                ('email', models.EmailField(help_text=b'Email will be used for send all exceptions from command.', max_length=75)),
                ('is_active', models.BooleanField(default=True, help_text=b'Email may be switched off for a little while.')),
            ],
            options={
                'verbose_name': 'Email',
                'verbose_name_plural': 'Emails',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MMCHost',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateField(auto_now=True)),
                ('name', models.CharField(unique=True, max_length=255)),
                ('ignore', models.BooleanField(default=False, help_text=b'All logs from all scripts on this host will be ignored.')),
            ],
            options={
                'verbose_name': 'Host',
                'verbose_name_plural': 'Hosts',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MMCLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateField(auto_now=True)),
                ('start', models.DateTimeField()),
                ('end', models.DateTimeField(auto_now=True)),
                ('elapsed', models.FloatField()),
                ('success', models.NullBooleanField(default=None)),
                ('error_message', models.TextField(null=True, blank=True)),
                ('traceback', models.TextField(null=True, blank=True)),
                ('sys_argv', models.CharField(max_length=255, null=True, blank=True)),
                ('memory', models.FloatField(default=0.0)),
                ('cpu_time', models.FloatField(default=0.0)),
                ('hostname', models.ForeignKey(to='mmc.MMCHost')),
            ],
            options={
                'verbose_name': 'Log',
                'verbose_name_plural': 'Logs',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MMCScript',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateField(auto_now=True)),
                ('name', models.CharField(unique=True, max_length=255)),
                ('ignore', models.BooleanField(default=False, help_text=b'All logs from this script will be ignored.')),
                ('one_copy', models.BooleanField(default=False, help_text=b'Only one copy of this script will be run.')),
                ('save_on_error', models.BooleanField(default=False, help_text=b'This flag used only for ignored commands.')),
            ],
            options={
                'verbose_name': 'Script',
                'verbose_name_plural': 'Scripts',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='mmclog',
            name='script',
            field=models.ForeignKey(to='mmc.MMCScript'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0014_mmcscript_temporarily_disabled'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mmcemail',
            name='ignore',
            field=models.ManyToManyField(help_text=b'Helpful for different teams. Choose script which you want to ignore.', to='mmc.MMCScript', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0013_mmclog_is_fixed'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='temporarily_disabled',
            field=models.BooleanField(default=False, help_text=b'Temporarily disable script execution'),
        ),
    ]



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'MMCScript.one_copy'
        db.add_column('mmc_mmcscript', 'one_copy',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'MMCScript.one_copy'
        db.delete_column('mmc_mmcscript', 'one_copy')


    models = {
        'mmc.mmcemail': {
            'Meta': {'object_name': 'MMCEmail'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'mmc.mmchost': {
            'Meta': {'object_name': 'MMCHost'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ignore': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'mmc.mmclog': {
            'Meta': {'object_name': 'MMCLog'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'elapsed': ('django.db.models.fields.FloatField', [], {}),
            'end': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'error_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hostname': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['mmc.MMCHost']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'script': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['mmc.MMCScript']"}),
            'start': ('django.db.models.fields.DateTimeField', [], {}),
            'success': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'sys_argv': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'traceback': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'mmc.mmcscript': {
            'Meta': {'object_name': 'MMCScript'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ignore': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'one_copy': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['mmc']


__author__ = 'gotlium'

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    @staticmethod
    def ping(address):
        import os

        os.system('ping -c 4 %s' % address)

    def handle(self, *args, **options):
        use_subprocess = True
        self.run_at_subprocess(use_subprocess, self.ping, '8.8.8.8')
        print("Done")



__author__ = 'gotlium'

from django.utils import timezone
from django.db import models
try:
    from django.utils.importlib import import_module
except ImportError:
    from importlib import import_module

from mmc.defaults import SUBJECT, MAIL_MODULE, EMAIL_FROM, DEFAULT_ONE_COPY
from mmc import python_2_unicode_compatible


@python_2_unicode_compatible
class MMCHost(models.Model):
    created = models.DateField(auto_now=True)
    name = models.CharField(max_length=255, unique=True)
    ignore = models.BooleanField(
        default=False,
        help_text='All logs from all scripts on this host will be ignored.')

    class Meta:
        verbose_name = 'Host'
        verbose_name_plural = 'Hosts'

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class MMCScript(models.Model):
    created = models.DateField(auto_now=True)
    name = models.CharField(max_length=255, unique=True)
    ignore = models.BooleanField(
        default=False, help_text='All logs from this script will be ignored.')
    one_copy = models.BooleanField(
        default=DEFAULT_ONE_COPY,
        help_text='Only one copy of this script will be run.')
    save_on_error = models.BooleanField(
        default=False,
        help_text='This flag can be used only for ignored commands.')
    real_time = models.BooleanField(
        default=False,
        help_text='Real Time info about command (for long tasks).')
    interval_restriction = models.PositiveIntegerField(
        default=None, null=True, blank=True,
        help_text='Interval before script can be run again')
    calls = models.BigIntegerField('Number of calls', default=0)
    enable_queries = models.BooleanField(
        default=False, help_text='Profile queries numbers.')
    trigger_cpu = models.FloatField(
        null=True, blank=True, help_text='Set the threshold time for CPU')
    trigger_memory = models.FloatField(
        null=True, blank=True, help_text='Set the threshold MB for Memory')
    trigger_time = models.FloatField(
        null=True, blank=True,
        help_text='Set the threshold sec for execution time')
    trigger_queries = models.FloatField(
        null=True, blank=True,
        help_text='Set the threshold number of queries')
    enable_triggers = models.BooleanField(
        default=False, help_text='Enable triggers for receive email '
                                 'notification, if threshold of counters '
                                 'will be exceeded')
    temporarily_disabled = models.BooleanField(
        default=False, help_text='Temporarily disable script execution')

    def update_calls(self):
        MMCScript.objects.filter(pk=self.pk).update(
            calls=models.F('calls') + 1)

    @classmethod
    def get_one_copy(cls, name):
        return cls.objects.get(name=name).one_copy

    @classmethod
    def run_is_allowed(cls, name):
        script = cls.objects.get(name=name)
        interval_restriction = script.interval_restriction

        if not interval_restriction:
            return True

        logs = MMCLog.objects.filter(script=script).order_by('-id')[:1]
        if logs.exists():
            if (timezone.now() - logs[0].start).seconds < interval_restriction:
                return False
        return True

    @classmethod
    def run_is_enabled(cls, name):
        script = cls.objects.get(name=name)

        return not script.temporarily_disabled

    class Meta:
        verbose_name = 'Script'
        verbose_name_plural = 'Scripts'

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class MMCLog(models.Model):
    created = models.DateField(auto_now=True)
    start = models.DateTimeField()
    end = models.DateTimeField(auto_now=True)
    elapsed = models.FloatField()
    hostname = models.ForeignKey(MMCHost)
    script = models.ForeignKey(MMCScript)
    success = models.NullBooleanField(default=None)
    error_message = models.TextField(blank=True, null=True)
    traceback = models.TextField(blank=True, null=True)
    sys_argv = models.CharField(max_length=255, blank=True, null=True)
    memory = models.FloatField(default=0.00)
    cpu_time = models.FloatField(default=0.00)
    was_notified = models.BooleanField(default=False)
    stdout_messages = models.TextField(blank=True, null=True)
    pid = models.IntegerField(default=1)
    queries = models.BigIntegerField(default=0)
    is_fixed = models.NullBooleanField(default=None)

    def __str__(self):
        return self.script.name

    class Meta:
        verbose_name = 'Log'
        verbose_name_plural = 'Logs'

    @classmethod
    def logging(cls, **kwargs):
        kwargs['script'] = MMCScript.objects.get_or_create(
            name=kwargs['script'])[0]
        kwargs['hostname'] = MMCHost.objects.get_or_create(
            name=kwargs['hostname'])[0]
        instance = kwargs.pop('instance', None)

        def do_save():
            if instance is not None:
                return cls.objects.filter(pk=instance.pk).update(**kwargs)
            kwargs.get('script').update_calls()
            return cls.objects.create(**kwargs)

        if not kwargs['script'].ignore and not kwargs['hostname'].ignore:
            return do_save()

        if kwargs['success'] is False and kwargs['script'].save_on_error:
            return do_save()

    @classmethod
    def get_lock_time(cls, script_name):
        return round(int(cls.objects.values_list(
            'elapsed', flat=True
        ).filter(script__name=script_name).order_by('-elapsed')[:1][0]) + 1)


@python_2_unicode_compatible
class MMCEmail(models.Model):
    created = models.DateField(auto_now=True, editable=False)
    email = models.EmailField(
        help_text='Email will be used for send all exceptions from command.')
    ignore = models.ManyToManyField(
        MMCScript, blank=True,
        help_text='Helpful for different teams. '
                  'Choose script which you want to ignore.')
    is_active = models.BooleanField(
        default=True,
        help_text='Email may be switched off for a little while.')

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = 'Email'
        verbose_name_plural = 'Emails'

    @classmethod
    def send(cls, host_name, script_name, message, subject=SUBJECT):
        try:
            emails = list(cls.objects.values_list(
                'email', flat=True
            ).filter(is_active=True).exclude(ignore__name=script_name))
            if emails:
                subject = subject % dict(script=script_name, host=host_name)

                mail = import_module(MAIL_MODULE)
                mail.send_mail(
                    subject, message, EMAIL_FROM, emails, fail_silently=True
                )
        except Exception as err:
            print("[MMC] {0}".format(err))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0009_auto_20150425_1540'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='enable_queries',
            field=models.BooleanField(default=False, help_text=b'Profile queries numbers'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0008_mmcscript_real_time'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmclog',
            name='queries',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='mmcscript',
            name='save_on_error',
            field=models.BooleanField(default=False, help_text=b'This flag can be used only for ignored commands.'),
        ),
    ]



__author__ = 'gotlium'

import tempfile
import datetime
import random
import time
import sys
import os

from mmc import defaults


class AbstractLock(object):
    def __init__(self, script):
        self._script = script
        self._lock_time = self.get_lock_time()
        self._tempdir = tempfile.gettempdir()

    def get_lock_time(self):
        try:
            from mmc.models import MMCLog

            return MMCLog.get_lock_time(self._script)
        except:
            return defaults.DEFAULT_LOCK_TIME

    def get_lock_file(self):
        return os.path.join(self._tempdir, self._script + '.lock')

    def unlock(self):
        raise NotImplementedError

    def lock(self):
        raise NotImplementedError

    def is_run(self):
        raise NotImplementedError

    def check_is_running(self):
        if self.is_run():
            sys.stdout.write('Already running\n')
            sys.exit(-1)

    @staticmethod
    def random_wait():
        random.seed()
        time.sleep(random.choice([random.uniform(0, 3) for i in range(10)]))


class FileLock(AbstractLock):
    def unlock(self):
        if self.is_run():
            os.unlink(self.get_lock_file())

    def lock(self):
        open(self.get_lock_file(), 'w').close()

    def _cleanup_lock(self):
        if os.path.exists(self.get_lock_file()):
            created = datetime.datetime.fromtimestamp(
                os.path.getctime(self.get_lock_file()))
            seconds = (datetime.datetime.now() - created).seconds
            if seconds > self._lock_time:
                os.unlink(self.get_lock_file())

    def is_run(self):
        self._cleanup_lock()
        return os.path.exists(self.get_lock_file())


class RedisLock(AbstractLock):
    def __init__(self, script):
        from redis import StrictRedis

        self._cli = StrictRedis(**defaults.REDIS_CONFIG)
        super(RedisLock, self).__init__(script)
        self.random_wait()

    def unlock(self):
        if self.is_run():
            self._cli.delete(self.get_lock_file())

    def lock(self):
        return self._cli.set(self.get_lock_file(), '1', self._lock_time)

    def is_run(self):
        return self._cli.get(self.get_lock_file())


class MemcacheLock(RedisLock):
    def __init__(self, script):
        try:
            from memcache import Client
        except ImportError:
            from pylibmc import Client

        super(MemcacheLock, self).__init__(script)
        self._cli = Client(**defaults.MEMCACHED_CONFIG)
        self.random_wait()


def get_lock_instance(script):
    return globals().get(defaults.LOCK_TYPE)(script)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0010_mmcscript_enable_queries'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='trigger_queries',
            field=models.FloatField(
                help_text=b'Set the threshold number of queries',
                null=True, blank=True),
        ),
    ]



__author__ = 'gotlium'

import inspect
import mmc


class MonkeyProxy(object):
    def __init__(self, cls):
        monkey_bases = tuple(
            b._no_monkey for b in cls.__bases__ if hasattr(b, '_no_monkey'))
        for monkey_base in monkey_bases:
            for name, value in monkey_base.__dict__.iteritems():
                setattr(self, name, value)


def monkey_mix(cls, mixin):
    cls._no_monkey = MonkeyProxy(cls)
    is_bound_method = inspect.isfunction if mmc.PY3 else inspect.ismethod
    methods = inspect.getmembers(mixin, is_bound_method)

    for name, method in methods:
        if hasattr(cls, name):
            setattr(cls._no_monkey, name, getattr(cls, name))
        setattr(cls, name, method.im_func if mmc.PY2 else method)



# -*- encoding: utf-8 -*-

__author__ = 'gotlium'

from socket import gethostname
from django.conf import settings


def get_settings(key, default):
    return getattr(settings, key, default)


SUBJECT = get_settings('MMC_SUBJECT', '[MMC] Error: %(host)s: %(script)s')
SUBJECT_LIMIT = get_settings(
    'MMC_SUBJECT_LIMIT', '[MMC] Limit exceeded: %(host)s: %(script)s')

REDIS_CONFIG = get_settings('MMC_REDIS_CONFIG', {
    'host': 'localhost', 'port': 6379,
    'db': 0, 'password': None
})
MEMCACHED_CONFIG = get_settings('MMC_MEMCACHED_CONFIG', {
    'servers': ['127.0.0.1:11211'], 'debug': 0
})
LOCK_TYPE = get_settings('MMC_LOCK_TYPE', 'FileLock')

SENTRY_NOTIFICATION = get_settings('MMC_SENTRY_NOTIFICATION', True)
EMAIL_NOTIFICATION = get_settings('MMC_EMAIL_NOTIFICATION', True)
DEFAULT_LOCK_TIME = get_settings('MMC_DEFAULT_LOCK_TIME', 3600)
CLEANUP_IGNORED = get_settings('MMC_CLEANUP_IGNORED', True)
FORCE_IGNORE = get_settings('MMC_FORCE_IGNORE', ['shell'])
READ_STDOUT = get_settings('MMC_READ_STDOUT', False)
MAIL_MODULE = get_settings('MMC_MAIL_MODULE', 'django.core.mail')
EMAIL_FROM = get_settings('MMC_EMAIL_FROM', settings.DEFAULT_FROM_EMAIL)
REAL_TIME_UPDATE = get_settings('MMC_REAL_TIME_UPDATE', 3)
HOSTNAME = get_settings('MMC_HOSTNAME', gethostname())
DEFAULT_ONE_COPY = get_settings('MMC_DEFAULT_ONE_COPY', False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0004_mmcemail_ignore'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmcscript',
            name='enable_triggers',
            field=models.BooleanField(default=False, help_text=b'Enable triggers for receive email notification, if threshold of counters will be exceeded'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='mmcscript',
            name='trigger_cpu',
            field=models.FloatField(help_text=b'Set the threshold time for CPU', null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='mmcscript',
            name='trigger_memory',
            field=models.FloatField(help_text=b'Set the threshold MB for Memory', null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='mmcscript',
            name='trigger_time',
            field=models.FloatField(help_text=b'Set the threshold sec for execution', null=True, blank=True),
            preserve_default=True,
        ),
    ]
